#!/usr/bin/python3

####################################################################################################################
##### Script that manages actions on the ECX instance
#####
##### Syntax: actions_manager.py <stopecx | startecx | shutdown>
##### Example: actions_manager.py startecx
#####
####################################################################################################################

import subprocess
import string
import random
import os, shutil, logging, sys, re
from optparse import OptionParser

CONST = None
FORWARD_SLASH="/"

class LessThenFilter(logging.Filter):
    def __init__(self, level):
        self._level = level
        logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

def main(argv):

    log = logging.getLogger()
    log.setLevel(logging.NOTSET)

    sh_out = logging.StreamHandler(sys.stdout)
    sh_out.setLevel(logging.INFO)
    sh_out.setFormatter(logging.Formatter('%(message)s'))
    sh_out.addFilter(LessThenFilter(logging.WARNING))
    log.addHandler(sh_out)

    sh_err = logging.StreamHandler(sys.stderr)
    sh_err.setLevel(logging.WARNING)
    sh_err.setFormatter(logging.Formatter('%(message)s'))
    log.addHandler(sh_err)

    # create a file handler
    fhandler = logging.FileHandler(os.path.join('/data/log/adminconsole','actions-manager.log'))
    fhandler.setLevel(logging.DEBUG)
    fhandler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
    log.addHandler(fhandler)

    if len(sys.argv) == 2:
        add_wide_gaps()
        logging.debug('##########################################################################################################')
        logging.debug('RUNNING ACTIONS MANAGER')
        logging.debug('Argument Length ' + str(len(argv)))
        logging.debug('Arguments: ' + str(argv))
        process(argv[1])
    elif len(sys.argv) == 3:
        add_wide_gaps()
        logging.debug('##########################################################################################################')
        logging.debug('RUNNING ACTIONS MANAGER')
        logging.debug('Argument Length ' + str(len(argv)))
        logging.debug('Arguments: ' + str(argv))
        process_two_args(argv[1], argv[2])
    else:
        logging.debug('Invalid number of arguments')
        logging.debug('Argument Length ' + str(len(argv)))
        logging.debug('Arguments: ' + str(argv))
        logging.debug('Syntax:  actions_manager.py <stopecx | startecx | statusecx | restart>')
        logging.debug('Example: actions_manager.py startecx')
        logging.debug('Example: actions_manager.py stopecx')
        logging.debug('Example: actions_manager.py statusecx')
        logging.debug('Example: actions_manager.py restart')
        failed_exit()

def add_wide_gaps():
    for num in range(1,5):
        logging.debug('')

def clean_exit():
    logging.debug('')
    logging.debug('Script completed successfully, Exiting!')
    logging.debug('')
    sys.exit(0)

def failed_exit():
    logging.debug('')
    logging.debug('Script completed with failures')
    sys.exit(1)

#################################################################
# List timezones on the appliance
#################################################################
def get_current_timezone():

    command = '/usr/bin/timedatectl | /usr/bin/head -n 4 | /usr/bin/tail -1 | /usr/bin/xargs'

    pobj = subprocess.Popen(command, bufsize=1, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd='/tmp')

    out, err = pobj.communicate()

    try:
        out = out.decode()
    except Exception as e:
        logging.info('Cannot decode the Output')

    if err:
        logging.debug('')
        logging.debug('Command Errors :')
        logging.debug('----------------')
        logging.debug('')
        logging.error(err)

        failed_exit()

    logging.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.info(out)

        clean_exit()
    else:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.error(out)

        failed_exit()

#################################################################
# Get timezones on the appliance
#################################################################
def get_timezones():

    command = '/usr/bin/timedatectl list-timezones'

    pobj = subprocess.Popen(command, bufsize=1, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd='/tmp')

    out, err = pobj.communicate()

    try:
        out = out.decode()
    except Exception as e:
        logging.info('Cannot decode the Output')

    if err:
        logging.debug('')
        logging.debug('Command Errors :')
        logging.debug('----------------')
        logging.debug('')
        logging.error(err)

        failed_exit()

    logging.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.info(out)

        clean_exit()
    else:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.error(out)

        failed_exit()


#################################################################
# Change timezone of the appliance
#################################################################
def change_timezone(timezone):

    command = '/usr/bin/timedatectl set-timezone ' + timezone

    pobj = subprocess.Popen(command, bufsize=1, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd='/tmp')

    out, err = pobj.communicate()

    if err:
        logging.debug('')
        logging.debug('Command Errors :')
        logging.debug('----------------')
        logging.debug('')
        logging.error(err)

        failed_exit()

    logging.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.info(out)

        clean_exit()
    else:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.error(out)

        failed_exit()

#################################################################
# Process Update Check
#################################################################
def process_action(mode):

    if mode == "startapp":
        command = '/usr/bin/systemctl start virgo'
    elif mode == "stopapp":
        command = '/usr/bin/systemctl stop virgo'
    elif mode == "restartapp":
        command = '/usr/bin/systemctl restart virgo'
    elif mode == "appstatus":
        command = '/usr/bin/systemctl status virgo'
    elif mode == "restart":
        command = '(sleep 5; shutdown -r now) &'

    logging.debug('Command: ' + command)

    pobj = subprocess.Popen(command, bufsize=1, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd='/tmp')

    if mode == "restart":
        clean_exit()

    out, err = pobj.communicate()

    logging.debug('Command Returncode : ' + str(pobj.returncode))

    if err:
        logging.debug('')
        logging.debug('Command Errors :')
        logging.debug('----------------')
        logging.debug('')
        logging.error(err)

    if pobj.returncode == 0:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.info(out)

        clean_exit()
    else:
        if out:
            logging.debug('')
            logging.debug('Command Output :')
            logging.debug('----------------')
            logging.debug('')
            logging.error(out)

        failed_exit()

def process_two_args(param1, param2):
    if param1 == "changetimezone":
        logging.debug('Change TimeZone')
        change_timezone(param2)

def process(mode):
    if mode == "startapp":
        logging.debug('Starting product')
        process_action(mode)
    elif mode == "stopapp":
        logging.debug('Stopping product')
        process_action(mode)
    elif mode == "appstatus":
        logging.debug('Checking product status')
        process_action(mode)
    elif mode == "restartapp":
        logging.debug('Restarting product')
        process_action(mode)
    elif mode == "restart":
        logging.debug('Restarting instance')
        process_action(mode)
    elif mode == "gettimezones":
        logging.debug('Get Timezones')
        get_timezones()
    elif mode == "getcurrenttimezone":
        logging.debug('Get Current Timezone')
        get_current_timezone()
    else:
        logging.debug('Unknown mode, please check usage')
        failed_exit()

if __name__ == "__main__":
    main(sys.argv)
