#!/usr/bin/python3

####################################################################################################################
##### Script runs System Admin product information tasks
#####
##### Syntax: productinfo-manager.py <check|update>
##### Example: productinfo-manager.py update
#####
####################################################################################################################

from subprocess import call ,Popen, PIPE
import time
import string
import random
import os, shutil, logging, sys, re
from optparse import OptionParser, SUPPRESS_HELP
import urllib.request, urllib.parse, urllib.error

CONST = None
FORWARD_SLASH="/"

class LessThenFilter(logging.Filter):
    def __init__(self, level):
        self._level = level
        logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

def setup():
    global logger
    logger = logging.getLogger('productinfo_manager')
    logger.setLevel(logging.DEBUG)

    # create a console handler
    cHandler = logging.StreamHandler(sys.stdout)
    cHandler.setLevel(logging.INFO)
    cHandler.setFormatter(logging.Formatter('%(message)s'))
    cHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(cHandler)

    # create a console handler for stderr
    ceHandler = logging.StreamHandler()
    ceHandler.setLevel(logging.WARNING)
    ceHandler.setFormatter(logging.Formatter('%(message)s'))
    logger.addHandler(ceHandler)

    # create a file handler
    fHandler = logging.FileHandler(options.log)
    fHandler.setLevel(logging.DEBUG)
    fHandler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
    logger.addHandler(fHandler)

    add_wide_gaps()
    logger.debug('+--------------------------------------------------------------------------------+')
    logger.debug('+----------------------- STARTING PRODUCTINFO MANAGER ---------------------------+')
    logger.debug('')
    logger.debug('Current local time: ' + time.asctime(time.localtime(time.time())) );
    logger.debug('')
    logger.debug(os.path.realpath(__file__) + ' called with options: ' + options.__str__())

def main():

    group = repo.get_group_name()

    if options.mode =='getpackagesinfo': get_packages_info(group)

    if options.mode =='getproductinfo': get_product_info(group)

    if options.mode == 'gethotfixes': gethotfixes(group)

    if options.mode == 'gethotfixchangelog': gethotfix_changelog(options.package)

    if options.mode == 'deployhotfix': deploy_hotfix(options.location)

def add_wide_gaps():
    for num in range(1,5):
        logger.debug('')

def clean_exit():
    logger.debug('')
    logger.debug('Script completed successfully, Exiting!')
    logger.debug('')
    sys.exit(0)

def failed_exit():
    logger.debug('')
    logger.debug('Script completed with failures ......')
    sys.exit(1)

#######################################
# Get product info based on RPM package
# arguments: group
#######################################
def get_product_info(group):

    logger.debug('Getting product info for group: ' + group )

    command = '/bin/rpm -qa |  /bin/grep ' + group + '-*-[0-9].[0-9].[0-9]'

    logger.debug('Command: ' + command)

    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
    out, err = pobj.communicate()

    logger.debug('Command Returncode : ' + str(pobj.returncode))

    process_command_output(pobj.returncode, out, err)

#############################################
# Get RPM package information for the product
# arguments: group
#############################################
def get_packages_info(group):

    logger.debug('Getting packages info for group: ' + group )

    if group == 'ecx':
        command = '/bin/rpm -qa --qf \"%{NAME} %{VERSION} %{RELEASE} %{ARCH}\\n\"| /bin/grep \'vsnap\|vadp\|' + group + '\''
    else:
        command = '/bin/rpm -qa --qf \"%{NAME} %{VERSION} %{RELEASE} %{ARCH}\\n\"| /bin/grep \'ecx\|vsnap\|vadp\|' + group + '\''

    logger.debug('Command: ' + command)

    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
    out, err = pobj.communicate()

    try:
        out = out.decode()
    except Exception as e:
        logger.info('Cannot decode the Output')

    logger.debug('Command Returncode : ' + str(pobj.returncode))

    process_command_output(pobj.returncode, out, err)

##########################################
## Get currently installed hotfix packages
##########################################
def gethotfixes(group):

    logger.debug('Getting installed ' + group + ' hotfix packages')

    command = '/bin/rpm -qa | /bin/grep \'' + group + '-efix-*\''

    logger.debug('Command: ' + command)

    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
    out, err = pobj.communicate()

    logger.debug('Command Returncode : ' + str(pobj.returncode))

    process_command_output(pobj.returncode, out, err)


####################################################
## Gets change log information for an hotfix package
####################################################
def gethotfix_changelog(package):
    logger.debug('Getting change log information for ' + package + ' hotfix package')

    command = '/bin/rpm -q --changelog ' + package

    logger.debug('Command: ' + command)

    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
    out, err = pobj.communicate()

    logger.debug('Command Returncode : ' + str(pobj.returncode))

    process_command_output(pobj.returncode, out, err)


########################
## Deploy hotfix package
########################
def deploy_hotfix(hotfix_package_path):

    if os.path.exists(hotfix_package_path):
        logger.debug('Deploy hotfix package ' + hotfix_package_path)

        command = '/bin/rpm -i ' + hotfix_package_path

        logger.debug('Command: ' + command)

        pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
        out, err = pobj.communicate()

        logger.debug('Command Returncode : ' + str(pobj.returncode))

        process_command_output(pobj.returncode, out, err)
    else:
        logger.error('Hotfix package ' + hotfix_package_path + ' does not exist')
        failed_exit()


#########################
## Process command output
#########################
def process_command_output(returncode, out, err):

    if returncode == 0:
        if out:
            logger.debug('')
            logger.debug('Command Output :')
            logger.debug('----------------')
            logger.debug('')
            logger.info(out)
            clean_exit()
    else:
        if out:
            logger.debug('')
            logger.debug('Command Output :')
            logger.debug('----------------')
            logger.debug('')
            logger.error(out)

        if err:
            logger.debug('')
            logger.debug('Command Errors :')
            logger.debug('----------------')
            logger.debug('')
            logger.error(err)

        failed_exit()

#########################
# Represents product repo
#########################
class Repo:
    'Class for repo type'

    def __init__(self,group_name='ecx'):
        self.group_name  = group_name

    def get_group_name(self):
        return self.group_name


if __name__ == "__main__":

    parser = OptionParser(version='%prog version 1.0')

    usage = "usage: %prog [options]"
    parser = OptionParser(usage=usage)

    parser.add_option("--version", help="version")

    parser.add_option("-l","--log",
                  default='/data/log/adminconsole/productinfo-manager.log',
                  dest='log',
                  help="Log file location"
                  " [ default: %default ]" )

    parser.add_option("-v", action="count", dest="verbosity", help=SUPPRESS_HELP)

    parser.add_option("--mode",
                  help="Supported modes for hotfix manager: getpackagesinfo, getproductinfo,  gethotfixes, gethotfixchangelog, deployhotfix")

    parser.add_option("--package",
                  help="Name of hotfix package, needs to be used with \'mode\' \'gethotfixchangelog\'")

    parser.add_option("--location",
                  help="Location of hotfix package, needs to be used with \'mode\' \'deployhotfix\'")

    (options, args) = parser.parse_args()

    setup()

    if options.mode is None:
        logger.error('')
        logger.error('Mandatory option \'mode\' is missing')
        parser.print_help()
        failed_exit()

    if options.mode != 'gethotfixes' and options.mode != 'gethotfixchangelog' and options.mode != 'deployhotfix' and options.mode != 'getpackagesinfo' and options.mode != 'getproductinfo':
        logger.error('')
        logger.error('Invalid \'mode\' specified')
        parser.print_help()
        sys.exit(1)

    if options.mode == 'gethotfixchangelog' and not options.package:
        logger.error('')
        logger.error('Mode \'gethotfixchangelog\' requires package option')
        parser.print_help()
        sys.exit(1)

    if options.mode == 'deployhotfix' and not options.location:
        logger.error('')
        logger.error('Mode \'deployhotfix\' requires location option')
        parser.print_help()
        sys.exit(1)

    if os.path.exists( '/etc/yum.repos.d/catalogic.repo'): repo=Repo('ecx' )
    elif os.path.exists( '/etc/yum.repos.d/spp.repo'):    repo=Repo('spp')
    else:
        repo = None
        logger.error('Can not detect product type')
        failed_exit()

    main()
